<?php
/*
@wordpress-plugin
Plugin Name: XWay integration for WC
Description: Интеграция магазина на WooCommerce с Aliexpress, Ozon, Wildberries
Plugin URI: 
Version: 0.9.0
Requires at least: 5.7
Requires PHP: 7.0
Author: IIOXOTXOX (d.beloshapko@xway.ru)
Author URI:  	https://xway.ru/
License:	 	GPL-2.0+
License URI:	http://www.gnu.org/licenses/gpl-2.0.txt
Text Domain:	xway-integration
*/

if (!defined('ABSPATH')) exit;

function xway_ali_start()
{
	include 'page-settings.php';
}

function xway_products()
{
	include 'page-products.php';
}

function xway_yml()
{
	include 'page-yml.php';
}


add_action('admin_init', 'xway_ali_register_settings');
function xway_ali_register_settings()
{
	register_setting('xway-settings-group', 'xway_token');
	register_setting('xway-settings-group', 'xway_shop_id');
	register_setting('xway-settings-xml-group', 'xway_yml_shopname');
	register_setting('xway-settings-xml-group', 'xway_yml_company');
}


add_action('activated_plugin', 'xway_activation_redirect');
function xway_activation_redirect($plugin)
{
	if ($plugin == plugin_basename(__FILE__)) {
		exit(wp_redirect(admin_url('admin.php?page=xway-settigns')));
	}
}


register_deactivation_hook(__FILE__, 'xway_ali_plugin_deactivation');
function xway_ali_plugin_deactivation()
{
	delete_option('xway_token');
	delete_option('xway_shop_id');
	delete_option('xway_yml_shopname');
	delete_option('xway_yml_company');
}


if (!class_exists('XWAY_YML_generator')) {
	class XWAY_YML_generator
	{

		private $settings;

		public function __construct()
		{
			$this->settings = get_option('xway_token');
			$this->xway_shop_id = get_option('xway_shop_id');
			$this->errors = isset($_SESSION['xway_error']) ? array_filter(wp_unslash((array)$_SESSION['xway_error'])) : array();
			header('Content-Type: text/xml');
			header('Content-Type: application/force-download; charset=UTF-8');
			header('Cache-Control: no-store, no-cache');
			header('Content-disposition: attachment; filename="' . 'YML products' . '.xml"');


			$args = array(
				'post_type' => 'product',
				'posts_per_page' => -1
			);

			$query = new WP_Query($args);

			$yml = '';
			$yml .= $this->yml_header(get_woocommerce_currency());
			$yml .= $this->yml_offers($query);
			$yml .= $this->yml_footer();
			print_r($yml);
			exit();
		}

		private function yml_header($currency)
		{

			$yml  = '<?xml version="1.0" encoding="utf-8"?>' . PHP_EOL;
			$yml .= '<!DOCTYPE yml_catalog SYSTEM "shops.dtd">' . PHP_EOL;
			$yml .= '<yml_catalog date="' . date('Y-m-d H:i') . '">' . PHP_EOL;
			$yml .= '  <shop>' . PHP_EOL;
			$yml .= '    <name>' . $this->specialchars_encode($this->settings['shop']['name']) . '</name>' . PHP_EOL;
			$yml .= '    <company>' . $this->specialchars_encode($this->settings['shop']['company']) . '</company>' . PHP_EOL;
			$yml .= '    <url>' . get_site_url() . '</url>' . PHP_EOL;
			$yml .= '    <currencies>' . PHP_EOL;

			if (('USD' === $currency) || ('EUR' === $currency)) {
				$yml .= '      <currency id="RUR" rate="1"/>' . PHP_EOL;
				$yml .= '      <currency id="' . $currency . '" rate="СВ" />' . PHP_EOL;
			} else {
				$yml .= '      <currency id="' . $currency . '" rate="1" />' . PHP_EOL;
			}

			$yml .= '    </currencies>' . PHP_EOL;
			$yml .= '    <categories>' . PHP_EOL;

			$args = [
				'taxonomy' => 'product_cat',
				'orderby'  => 'term_id',
			];

			if (isset($this->settings['include_cat'])) {
				$args['include'] = array_column($this->settings['include_cat'], 'value');
			}

			foreach (get_categories($args) as $category) {
				if (0 === $category->parent) {
					$yml .= '      <category id="' . $category->cat_ID . '">' . wp_strip_all_tags($category->name) . '</category>' . PHP_EOL;
				} else {
					$yml .= '      <category id="' . $category->cat_ID . '" parentId="' . $category->parent . '">' . wp_strip_all_tags($category->name) . '</category>' . PHP_EOL;
				}
			}

			$yml .= '    </categories>' . PHP_EOL;
			$yml .= '    <offers>' . PHP_EOL;

			return $yml;
		}

		private function yml_footer()
		{

			$yml  = '    </offers>' . PHP_EOL;
			$yml .= '  </shop>' . PHP_EOL;
			$yml .= '</yml_catalog>' . PHP_EOL;

			return $yml;
		}

		private function yml_offers(WP_Query $query)
		{

			global $product, $offer;

			$yml = '';

			while ($query->have_posts()) {
				$query->the_post();
				$product = wc_get_product(get_the_ID());
				if (apply_filters('me_exclude_post', false, $query->post, $product)) {
					continue;
				}
				// We use a separate variable for offer because we will be rewriting it for variable products.
				$offer = $product;
				/**
				 * By default we set $variation_count to 1.
				 * That means that there is at least one product available.
				 * Variation products will have more than 1 count.
				 */
				$variations      = [];
				$variation_count = 1;
				if ($product->is_type('variable')) {
					$variations      = $product->get_available_variations();
					$variation_count = count($variations);
				}
				while ($variation_count > 0) {
					$variation_count--;

					if ($product->is_type('variable')) {
						$stock_status = $offer->is_in_stock();
					}

					// If variable product, get product id from $variations array.
					$offer_id = (($product->is_type('variable')) ? $variations[$variation_count]['variation_id'] : $product->get_id());
					if ($product->is_type('variable')) {
						if (!$offer->is_in_stock()) {
							continue;
						}

						// This has to work but we need to think of a way to save the initial offer variable.
						$offer = new WC_Product_Variation($offer_id);
					}

					/**
					 * We need to get typePrefix and model early to decide if the product is a vendor.model type or not.
					 *
					 * @since 2.0.4
					 */
					$vendor_model_type = $model = $type_prefix = false;

					// typePrefix.
					if (isset($this->settings['offer']['typePrefix']) && 'disabled' !== $this->settings['offer']['typePrefix']) {
						$type_prefix = $product->get_attribute('pa_' . $this->settings['offer']['typePrefix']);
						if ($type_prefix) {
							$vendor_model_type = true;
						}
					}

					// Model.
					if (isset($this->settings['offer']['model']) && 'disabled' !== $this->settings['offer']['model']) {
						$model = $product->get_attribute('pa_' . $this->settings['offer']['model']);
						if ($model) {
							$vendor_model_type = true;
						}
					}



					$post_image = get_the_post_thumbnail_url($offer->get_id(), 'full');
					if (!$post_image) {
						$post_image = get_the_post_thumbnail_url($product->get_id(), 'full');
					}
					$picture = '';
					if ($post_image !== false) {
						$picture = '        <picture>' . esc_url($post_image) . '</picture>' . PHP_EOL;
					}

					$gallery_ids = $product->get_gallery_image_ids();
					if (count($gallery_ids) > 9) {
						$gallery_ids = array_slice($gallery_ids, 0, 9);
					}

					foreach ($gallery_ids as $image_id) {
						if (false !== $image_url = wp_get_attachment_url($image_id)) {
							$picture .= '        <picture>' . esc_url($image_url) . '</picture>' . PHP_EOL;
						}
					}


					$params = '';

					foreach ($product->get_attributes() as $param) {
						$taxonomy = wc_attribute_taxonomy_name_by_id($param->get_id());

						if (isset($param['variation']) && true === $param['variation'] || isset($param['is_variation']) && 1 === $param['is_variation']) {
							$param_value = $offer->get_attribute($taxonomy);
						} else {
							$param_value = $product->get_attribute($taxonomy);
						}

						$attributes = explode(',', $param_value);
						foreach ($attributes as $single_param) {
							$attribute_name = wc_sanitize_taxonomy_name($taxonomy);
							$param_name  = apply_filters('me_param_name', wc_attribute_label($taxonomy),  0 === strpos($attribute_name, 'pa_') ? substr($attribute_name, 3) : $attribute_name);
							$param_value = apply_filters('me_param_value', trim($single_param));
							$param_unit  = apply_filters('me_param_unit', false, $param_name);

							$params .= '        <param name="' . $param_name . '" ' . $param_unit . '>' . $param_value . '</param>' . PHP_EOL;
						}
					}




					$yml .= '      <offer id="' . $offer_id . '" available="true" type="vendor.model">' . PHP_EOL;
					$yml .= '        <url>' . $this->specialchars_encode(get_permalink($offer->get_id())) . '</url>' . PHP_EOL;
					if ($offer->get_sale_price() && ($offer->get_sale_price() < $offer->get_regular_price())) {
						$yml .= '        <price>' . $offer->get_sale_price() . '</price>' . PHP_EOL;
						$yml .= '        <oldprice>' . $offer->get_regular_price() . '</oldprice>' . PHP_EOL;
					} else {
						$yml .= '        <price>' . (int)apply_filters('me_product_price', $offer->get_regular_price(), $offer->get_id()) . '</price>' . PHP_EOL;
					}
					$yml .= '        <currencyId>' . get_woocommerce_currency() . '</currencyId>' . PHP_EOL;
					$yml .= '        <categoryId>' . get_the_terms($product->get_id(), 'product_cat')[0]->term_id . '</categoryId>' . PHP_EOL;
					$yml .= $picture;
					$yml .= '        <name>' . $this->specialchars_encode(get_the_title($offer_id)) . '</name>' . PHP_EOL;
					$yml .= '        <param name="Name product ' . strtoupper(get_bloginfo('language')) . '">' . $this->specialchars_encode(get_the_title($offer_id)) . '</param>' . PHP_EOL;
					$yml .= '        <param name="reference">' . $offer->get_sku() . '</param>' . PHP_EOL;
					$yml .= '        <param name="Description">' . $this->specialchars_encode(get_the_content($offer_id)) . '</param>' . PHP_EOL;
					$yml .= '        <param name="weight" unit="gm">' . (float)$offer->get_weight() * 1000 . '</param>' . PHP_EOL;
					$yml .= '        <param name="height" unit="mm">' . (float)$offer->get_height() * 10 . '</param>' . PHP_EOL;
					$yml .= '        <param name="width" unit="mm">' . (float)$offer->get_width() * 10 . '</param>' . PHP_EOL;
					$yml .= '        <param name="length" unit="mm">' . (float)$offer->get_length() * 10 . '</param>' . PHP_EOL;
					$yml .= $params;
					$yml .= '        <description>' . $this->specialchars_encode(get_the_content($offer_id)) . '</description>' . PHP_EOL;
					$yml .= '        <pickup>false</pickup>' . PHP_EOL;
					$yml .= '        <delivery>true</delivery>' . PHP_EOL;
					$yml .= '        <country_of_origin />' . PHP_EOL;
					$yml .= '        <manufacturer_warranty>true</manufacturer_warranty>' . PHP_EOL;
					$yml .= '      </offer>' . PHP_EOL;
				}
			}

			return $yml;
		}

		private function specialchars_encode($string)
		{

			$string = str_replace('"', '&quot;', $string);
			$string = str_replace('&', '&amp;', $string);
			$string = str_replace('>', '&gt;', $string);
			$string = str_replace('<', '&lt;', $string);
			$string = str_replace('\'', '&apos;', $string);

			return $string;
		}
	}
}

function xway_yml_generator()
{
	if (isset($_POST['xway_generate_xml'])) {
		new XWAY_YML_generator();
		exit();
	}
}
add_action('admin_init', 'xway_yml_generator');

if (!class_exists('xway')) {
	class xway
	{

		public $xway_token;
		public $xway_shop_id;
		public $errors;

		public function __construct()
		{
			if (session_id() == '') {
				session_start();
			}
			$this->xway_token = get_option('xway_token');
			$this->xway_shop_id = get_option('xway_shop_id');
			$this->errors = isset($_SESSION['xway_error']) ? array_filter(wp_unslash((array)$_SESSION['xway_error'])) : array();
			load_plugin_textdomain('xway-integration', false, basename(dirname(__FILE__)) . '/languages/');
			add_action('admin_menu', array($this, 'xway_menu'));
			if (!$this->xway_check_wc()) {
				return false;
			}
			add_action('admin_notices', array($this, 'xway_error_save'), 99);

			add_action('woocommerce_reduce_order_stock', array($this, 'xway_change_stock'), 1, 1);
			add_action('woocommerce_restore_order_stock', array($this, 'xway_change_stock'), 1, 1);

			add_action('woocommerce_admin_order_totals_after_discount', array($this, 'xway_aliexpress_commission'), 99, 1);

			add_filter('woocommerce_order_get_total', array($this, 'ali_take_aliexpress_commission'), 1, 2);
			add_filter('woocommerce_admin_order_data_after_order_details', array($this, 'ali_track_number'), 1, 1);


			add_action('wp_ajax_xway_get_pdf_label', array($this, 'xway_get_pdf_label'));
			add_action('wp_ajax_nopriv_xway_get_pdf_label', array($this, 'xway_get_pdf_label'));


			add_filter('views_edit-shop_order', array($this, 'ali_views'), 1, 1);

			add_action('pre_get_posts', array($this, 'show_only_ali_orders'));

			add_action('save_post_product', array($this, 'xway_save_product_postdata'), 1, 3);
			add_action('save_post_shop_order', array($this, 'xway_save_shop_order_postdata'), 1, 3);

			add_action('init', array($this, 'register_awaiting_shipment_order_status'));
			add_filter('wc_order_statuses', array($this, 'add_awaiting_shipment_to_order_statuses'));


			add_action('init', array($this, 'ali_listen_api'), 999, 1);
		}

		public function xway_menu()
		{
			add_menu_page(
				'Xway',
				'Xway',
				'administrator',
				'xway-settigns',
				'xway_ali_start',
				'dashicons-cloud'
			);
			if (!$this->xway_check_wc()) {
				return false;
			}
			add_submenu_page(
				'xway-settigns',
				'Xway Products',
				__('Products', 'xway'),
				'administrator',
				'xway-products',
				'xway_products',
				1
			);
			add_submenu_page(
				'xway-settigns',
				'Xway YML',
				'YML',
				'administrator',
				'xway-yml',
				'xway_yml',
				2
			);
		}

		public function xway_check_wc($version = '6.4.0')
		{
			if (!function_exists('get_plugins')) {
				require_once ABSPATH . 'wp-admin/includes/plugin.php';
			}
			$_SESSION['xway_error'] = array();
			// Проверка активности плагина Woocommerce
			$plugin = 'woocommerce/woocommerce.php';
			if (!in_array($plugin, apply_filters('active_plugins', get_option('active_plugins', array()))) && !(is_multisite() && array_key_exists($plugin, get_site_option('active_sitewide_plugins', array())))) {
				array_push($_SESSION['xway_error'], 'Xway integration ' . __('requires WooCommerce installed and activated', 'xway'));
			}

			$woo_installed = get_plugins('/woocommerce');
			$woo_version   = $woo_installed['woocommerce.php']['Version'];

			// Проверка версии
			if (version_compare($woo_version, $version, '>=')) {
				return true;
			}

			return false;
		}

		function ali_listen_api()
		{
			if (preg_match('/\/xway\/orders\/add/', sanitize_url($_SERVER['REQUEST_URI']))) {
				header('Content-Type: application/json');
				$json = file_get_contents('php://input');
				$order = json_decode($json);
				$args = array(
					'meta_key' => 'xway_external_id',
					'meta_query' => array(
						array(
							'key' => 'xway_external_id',
							'compare' => 'NOT IN',
							'value' => array('')
						),
					),
				);
				$query = new WP_Query($args);

				$offer_ids = array();
				if ($query->posts) {
					foreach ($query->posts as $key => $post) {
						array_push($offer_ids, get_post_meta($post->ID, 'xway_external_id', true));
					}
				}

				$offer_exist = true;

				if ($this->xway_ali_order_exist($order->order_id)) {
					echo '{"success": true, "error":"order already exist"}';
					exit();
				}
				foreach ($order->order_items as $item) {
					if (!in_array($item->sku_code, $offer_ids)) {
						$offer_exist = false;
					}
				}
				if ($offer_exist) {
					$comment = '';
					switch ($order->order_status) {
						case 'WAIT_SELLER_SEND_GOODS':
						case 'WAIT_SELLER_EXAMINE_MONEY':
						case 'CONFIRMED':
							$status = 'wc-processing';
							break;
						case 'EQUIPED':
							$status = 'wc-equiped';
							break;
						case 'SELLER_PART_SEND_GOODS':
							$status = 'wc-part-send';
							break;
						case 'WAIT_BUYER_ACCEPT_GOODS':
						case 'IN_DELIVERY':
							$status = 'wc-awaiting';
							break;
						case 'IN_CANCEL':
						case 'REJECTED':
							$status = 'wc-cancelled';
							$comment = 'Отменён';
							break;
						case 'NOT_EQUIPED':
							$status = 'wc-cancelled';
							$comment = 'Отменён продавцом';
							break;
						case 'FUND_PROCESSING':
						case 'PLACE_ORDER_SUCCESS':
						case 'NEW':
							$status = 'wc-pending';
							break;
						case 'IN_ISSUE':
						case 'IN_FROZEN':
						case 'RISK_CONTROL':
							$status = 'wc-on-hold';
							break;
						case 'DELIVERED':
							$status = 'wc-completed';
							break;
						case 'FINISH': {
								switch (strtoupper($order->order_end_reason)) {
									case "SELLER_SEND_GOODS_TIMEOUT":
										$status = 'wc-failed';
										$comment = 'Время отправки вышло';
										break;
									case "CANCEL_ORDER_CLOSE_TRADE":
										$status = 'wc-cancelled';
										$comment = 'Заказ отменен клиентом';
										break;
									case "BUYER_ACCEPT_GOODS":
										$status = 'wc-completed';
										$comment = 'Клиент подтвердил получение';
										break;
									case "IN-SALE-RETURN-CLOSE-ORDER":
										$status = 'wc-refunded';
										$comment = 'Полный возврат товара и денег по спору';
										break;
									case "BUYER_CANCEL_NOTPAY_ORDER":
										$status = 'wc-cancelled';
										$comment = 'Покупатель отменил неоплаченный заказ';
										break;
									case "BUYER_PAY_TIMEOUT":
									case "buyer_pay_timeout":
										$status = 'wc-cancelled';
										$comment = 'Покупатель не оплатил заказ вовремя';
										break;
									case "SECURITY_CLOSE":
										$status = 'wc-cancelled';
										$comment = 'Закрыт системой безопасности АлиЭкспресс';
										break;
								}
								break;
							}
					}
					setlocale(LC_ALL, 'ru_RU', 'ru_RU.UTF-8', 'ru', 'russian');

					if ($user_id = email_exists($order->buyer_login_id . '@xway.ru')) {
						$user = $user_id;
					} else {
						$pass = wp_generate_password(6, false);
						$fullname = $order->buyer_name;
						$fullname = explode(' ', $fullname);
						$userdata = array(
							'user_login'    =>   $order->buyer_login_id . '@xway.ru',
							'user_email'    =>   sanitize_email($order->buyer_login_id . '@xway.ru'),
							'user_pass'     =>   $pass,
							'user_url'      =>   "",
							'first_name'    =>   $fullname[0],
							'last_name'     =>   $fullname[1],
							'display_name'	=>	 $order->buyer_signer_fullname,
							'nickname'      =>   $order->buyer_signer_fullname,
							'description'   =>   "",
							'role'			=> 	 "customer",
						);
						$user = wp_insert_user($userdata);
					}

					$new_order = wc_create_order(array(
						'status' => $status,
						'customer_id' => $user
					));

					$escrow_fee_rate = 0;
					$adjust_amount = 0;

					$logistics_service_name = $order->logistics_service_name;
					foreach ($order->order_items as $item) {

						$product_id = $this->get_post_by_external_id($item->sku_code); // a product ID or a variation ID
						if ((float)$item->lot_num == null || (float)$item->lot_num == 0) {
							$item->lot_num = 1;
						}
						$new_product_price = (float)$item->product_unit_price_amount / (float)$item->lot_num; // the new product price  <==== <==== <====
						$quantity = $item->product_count; // The line item quantity


						// Get an instance of the WC_Product object

						$product = wc_get_product($product_id);
						$product->set_price($new_product_price);
						$new_order->add_product($product, $quantity);


						$shipping = new WC_Order_Item_Shipping();

						if (empty($item->logistics_service_name) && !empty($logistics_service_name)) {
							$item->logistics_service_name = $logistics_service_name;
						}

						$shipping->set_props(array(
							'method_title' => $item->logistics_service_name,
							'method_id'    => '',
							'total'        => wc_format_decimal($item->logistics_amount),
							'taxes'        => array(),
							'order_id'     => $new_order->get_id(),
						));

						$shipping->save();

						$new_order->add_item($shipping);


						if ((float)$item->escrow_fee_rate > $escrow_fee_rate) {
							$escrow_fee_rate = (float)$item->escrow_fee_rate;
						}

						if (!empty($item->adjust_amount) && !is_null($item->adjust_amount)) {
							$adjust_amount += (float)$item->adjust_amount;
						}
					}

					if ($adjust_amount != 0) {
						$item_fee = new WC_Order_Item_Fee();

						$item_fee->set_name(__('Adjustment', 'xway')); // Generic fee name
						$item_fee->set_amount((float)$adjust_amount); // Fee amount
						$item_fee->set_tax_class(''); // default for ''
						$item_fee->set_tax_status('taxable'); // or 'none'
						$item_fee->set_total((float)$adjust_amount); // Fee amount
						$item_fee->calculate_taxes((float)$adjust_amount);

						$new_order->add_item($item_fee);
					}

					update_post_meta($new_order->get_id(), 'ali_commission', (float)('-' . $escrow_fee_rate));

					$new_order->calculate_totals();

					$new_order->save();

					if ($status == "wc-completed" || $status == "wc-processing" || $status == "wc-on-hold") {
						wc_reduce_stock_levels($new_order->get_id());
					}
					$post_id = $new_order->get_id();
					if (!empty($comment)) {
						$data = [
							'comment_post_ID'      => $post_id,
							'comment_content'      => $comment,
							'comment_type'         => 'order_note',
							'comment_parent'       => 0,
							'comment_date'         => null, // получим current_time('mysql')
							'comment_approved'     => 1,
						];

						wp_insert_comment(wp_slash($data));
					}
					if (is_numeric($post_id) && $post_id != false) {

						$fullname = $order->buyer_name;
						$fullname = explode(' ', $fullname);

						update_post_meta($post_id, '_billing_first_name', $fullname[0]);
						update_post_meta($post_id, '_billing_last_name', $fullname[1]);
						update_post_meta($post_id, '_billing_address_1', (empty($order->detail_address) && ($logistics_service_name == "Ozon Логистика" || $logistics_service_name == "доставка силами WB")) ? 'Order ID:' . $order->order_id : $order->detail_address);
						update_post_meta($post_id, '_billing_address_2', '');
						update_post_meta($post_id, '_billing_city', $order->city);
						update_post_meta($post_id, '_billing_state', $order->province);
						update_post_meta($post_id, '_billing_postcode', $order->zip);
						update_post_meta($post_id, '_billing_country', $order->country);
						update_post_meta($post_id, '_billing_email', '');
						update_post_meta($post_id, '_billing_phone', $order->phone_country . '' . $order->mobile_no);

						update_post_meta($post_id, '_shipping_first_name', $fullname[0]);
						update_post_meta($post_id, '_shipping_last_name', $fullname[1]);
						update_post_meta($post_id, '_shipping_address_1', $order->detail_address);
						update_post_meta($post_id, '_shipping_address_2', '');
						update_post_meta($post_id, '_shipping_city', $order->city);
						update_post_meta($post_id, '_shipping_state', $order->province);
						update_post_meta($post_id, '_shipping_postcode', $order->zip);
						update_post_meta($post_id, '_shipping_country', $order->country);
						update_post_meta($post_id, '_billing_email', '');
						update_post_meta($post_id, '_billing_phone', $order->phone_country . '' . $order->mobile_no);

						update_post_meta($post_id, '_order_currency', $order->symbol);
						update_post_meta($post_id, '_billing_address_index', $order->detail_address);
						update_post_meta($post_id, 'ali_order_id', $order->order_id);
						switch ((string)$order->marketplace_id) {
							case "1":
								update_post_meta($post_id, 'is_ali_order', 'yes');
								break;
							case "2":
								update_post_meta($post_id, 'is_ozon_order', 'yes');
								break;
							case "9":
								update_post_meta($post_id, 'is_wb_order', 'yes');
								break;
							default:
								update_post_meta($post_id, 'is_ali_order', 'yes');
								break;
						}


						echo '{"success": true, "comment":"Заказ ' . wp_kses($order->order_id, 'strip') . ' был создан в записе ' . wp_kses($post_id, 'strip') . '"}';;
						exit();
					}
					echo '{"success": false, "error":"$post_id not correct"}';
					exit();
				} else {
					echo '{"success": false, "error":"offer not exist"}';
					exit();
				}
				exit();
			} elseif (preg_match('/\/xway\/order\/update/', sanitize_url($_SERVER['REQUEST_URI']))) {

				header('Content-Type: application/json');
				$json = file_get_contents('php://input');
				$order = json_decode($json);
				if ($this->xway_ali_order_exist($order->order_id)) {
					$order_id = $this->get_order_by_ali_order_id($order->order_id);
					if (!$order_id) {
						echo '{"success": false, "error":"order don\'t found"}';
						exit();
					};
					$orderDetail = wc_update_order(array('order_id' => $order_id));

					$status = '';
					$comment = '';
					switch ($order->order_status) {
						case 'WAIT_SELLER_SEND_GOODS':
						case 'WAIT_SELLER_EXAMINE_MONEY':
						case 'CONFIRMED':
							$status = 'wc-processing';
							break;
						case 'EQUIPED':
							$status = 'wc-equiped';
							break;
						case 'SELLER_PART_SEND_GOODS':
							$status = 'wc-part-send';
							break;
						case 'WAIT_BUYER_ACCEPT_GOODS':
						case 'IN_DELIVERY':
							$status = 'wc-awaiting';
							break;
						case 'IN_CANCEL':
						case 'REJECTED':
							$status = 'wc-cancelled';
							$comment = 'Отменён';
							break;
						case 'NOT_EQUIPED':
							$status = 'wc-cancelled';
							$comment = 'Отменён продавцом';
							break;
						case 'FUND_PROCESSING':
						case 'PLACE_ORDER_SUCCESS':
						case 'NEW':
							$status = 'wc-pending';
							break;
						case 'IN_ISSUE':
						case 'IN_FROZEN':
						case 'RISK_CONTROL':
							$status = 'wc-on-hold';
							break;
						case 'DELIVERED':
							$status = 'wc-completed';
							break;
						case 'REFUND':
							$status = 'wc-refunded';
							break;
						case 'FINISH': {
								switch (strtoupper($order->order_end_reason)) {
									case "SELLER_SEND_GOODS_TIMEOUT":
										$status = 'wc-failed';
										$comment = 'Время отправки вышло';
										break;
									case "CANCEL_ORDER_CLOSE_TRADE":
										$status = 'wc-cancelled';
										$comment = 'Заказ отменен клиентом';
										break;
									case "BUYER_ACCEPT_GOODS":
										$status = 'wc-completed';
										$comment = 'Клиент подтвердил получение';
										break;
									case "IN-SALE-RETURN-CLOSE-ORDER":
										$status = 'wc-refunded';
										$comment = 'Полный возврат товара и денег по спору';
										break;
									case "BUYER_CANCEL_NOTPAY_ORDER":
										$status = 'wc-cancelled';
										$comment = 'Покупатель отменил неоплаченный заказ';
										break;
									case "BUYER_PAY_TIMEOUT":
									case "buyer_pay_timeout":
										$status = 'wc-cancelled';
										$comment = 'Покупатель не оплатил заказ вовремя';
										break;
									case "SECURITY_CLOSE":
										$status = 'wc-cancelled';
										$comment = 'Закрыт системой безопасности АлиЭкспресс';
										break;
								}
								break;
							}
					}

					$escrow_fee_rate = 0;
					$adjust_amount = 0;

					$args = array(
						'post_type' => 'product',
						'meta_key' => 'xway_external_id',
						'meta_query' => array(
							array(
								'key' => 'xway_external_id',
								'compare' => 'NOT IN',
								'value' => array('')
							),
						),
					);
					$query = new WP_Query($args);

					$offer_ids = array();
					if ($query->posts) {
						foreach ($query->posts as $key => $post) {
							array_push($offer_ids, get_post_meta($post->ID, 'xway_external_id', true));
						}
					}
					$offer_exist = true;

					foreach ($order->order_items as $item) {
						if (!in_array($item->sku_code, $offer_ids)) {
							$offer_exist = false;
						}
					}
					if ($offer_exist) {
						$product_list = array();
						$product_list_key = array();
						$order_items = $orderDetail->get_items();
						foreach ($order_items as $order_item_key => $order_item) {
							$product_id = $order_item->get_product_id();
							$product_list[] = $product_id;
							$product_list_key[] = $order_item_key;
						}
						$shipping_array = $orderDetail->get_items(array('shipping'));
						$shipping_list = array();
						foreach ($shipping_array as $shipping_value) {
							array_push($shipping_list, array(
								'name' => $shipping_value->get_name(),
								'total' => $shipping_value->get_total(),
							));
						}


						$fee_array = $orderDetail->get_items(array('fee'));
						$fee_list = array();
						foreach ($fee_array as $fee_key => $fee_value) {
							array_push($fee_list, array(
								'name' => $fee_value->get_name(),
								'total' => $fee_value->get_total(),
								'key' => $fee_key,
							));
						}

						$logistics_service_name = $order->logistics_service_name;
						foreach ($order->order_items as $item) {

							$product_id = $this->get_post_by_external_id($item->sku_code); // a product ID or a variation ID
							if (false !== $key = array_search($product_id, $product_list)) {
								if ((float)$item->lot_num == null || (float)$item->lot_num == 0) {
									$item->lot_num = 1;
								}
								$new_product_price = (float)$item->product_unit_price_amount / (float)$item->lot_num; // the new product price  <==== <==== <====
								$quantity = $item->product_count; // The line item quantity

								## - - - - - - - - - - - - - - - - - - - - - - - - - -  ##
								// Get an instance of the WC_Product object
								$product = $order_items[$product_list_key[$key]];
								// Change the product price
								$product->set_subtotal($new_product_price);
								$product->set_total($new_product_price);
								$product->set_quantity($quantity);

								$product->calculate_taxes();
								$product->save();

								if (empty($item->logistics_service_name) && !empty($logistics_service_name)) {
									$item->logistics_service_name = $logistics_service_name;
								}
								if (false !== $key = array_search($item->logistics_service_name, array_column($shipping_list, 'name'))) {
									if (wc_format_decimal($shipping_list[$key]['total']) != wc_format_decimal($item->logistics_amount)) {
										$shipping = new WC_Order_Item_Shipping();

										$shipping->set_props(array(
											'method_title' => $item->logistics_service_name,
											'method_id'    => '',
											'total'        => wc_format_decimal($item->logistics_amount),
											'taxes'        => array(),
											'order_id'     => $orderDetail->get_id(),
										));

										$shipping->save();

										$orderDetail->add_item($shipping);
									}
								} else {
									$shipping = new WC_Order_Item_Shipping();

									$shipping->set_props(array(
										'method_title' => $item->logistics_service_name,
										'method_id'    => '',
										'total'        => wc_format_decimal($item->logistics_amount),
										'taxes'        => array(),
										'order_id'     => $orderDetail->get_id(),
									));

									$shipping->save();

									$orderDetail->add_item($shipping);
								}


								if ((float)$item->escrow_fee_rate > $escrow_fee_rate) {
									$escrow_fee_rate = (float)$item->escrow_fee_rate;
								}

								if (!empty($item->adjust_amount)) {
									$adjust_amount += (float)$item->adjust_amount;
								}
							} else {
								if ((float)$item->lot_num == null || (float)$item->lot_num == 0) {
									$item->lot_num = 1;
								}
								$new_product_price = (float)$item->product_unit_price_amount / (float)$item->lot_num; // the new product price  <==== <==== <====
								$quantity = $item->product_count; // The line item quantity

								## - - - - - - - - - - - - - - - - - - - - - - - - - -  ##
								// Get an instance of the WC_Product object
								$product = wc_get_product($product_id);
								// Change the product price
								$product->set_price($new_product_price);
								$orderDetail->add_product($product, $quantity);


								$shipping = new WC_Order_Item_Shipping();
								if (empty($item->logistics_service_name) && !empty($logistics_service_name)) {
									$item->logistics_service_name = $logistics_service_name;
								}

								$shipping->set_props(array(
									'method_title' => $item->logistics_service_name,
									'method_id'    => '',
									'total'        => wc_format_decimal($item->logistics_amount),
									'taxes'        => array(),
									'order_id'     => $orderDetail->get_id(),
								));

								$shipping->save();

								$orderDetail->add_item($shipping);

								// Add Fee item to the order


								if ((float)$item->escrow_fee_rate > $escrow_fee_rate) {
									$escrow_fee_rate = (float)$item->escrow_fee_rate;
								}

								if (!empty($item->adjust_amount)) {
									$adjust_amount += (float)$item->adjust_amount;
								}
							}
						}

						if (false !== $key = array_search(__('Adjustment', 'xway'), array_column($fee_list, 'name'))) {
							if ($fee_list[$key]['total'] != wc_format_decimal((float)$adjust_amount)) {
								$fee = $fee_array[$fee_list[$key]['key']];
								$fee->set_total(wc_format_decimal((float)$adjust_amount));
								$fee->set_amount(wc_format_decimal((float)$adjust_amount));
								$fee->calculate_taxes((float)$adjust_amount);
							}
						} else {
							if ($adjust_amount != 0) {
								$item_fee = new WC_Order_Item_Fee();

								$item_fee->set_name(__('Adjustment', 'xway')); // Generic fee name
								$item_fee->set_amount((float)$adjust_amount); // Fee amount
								$item_fee->set_tax_class(''); // default for ''
								$item_fee->set_tax_status('taxable'); // or 'none'
								$item_fee->set_total((float)$adjust_amount); // Fee amount
								$item_fee->calculate_taxes((float)$adjust_amount);

								$orderDetail->add_item($item_fee);
							}
						}

						update_post_meta($orderDetail->get_id(), 'ali_commission', (float)('-' . $escrow_fee_rate));

						$orderDetail->calculate_totals();

						if ($status == "wc-completed" || $status == "wc-processing" || $status == "wc-on-hold") {
							wc_reduce_stock_levels($orderDetail->get_id());
						}
						if (!empty($status) && $orderDetail->get_status() != $status) {	//Проверка что статус изменился, проверка что новый статус не пустой
							$orderDetail->update_status($status, $comment, TRUE);
						}
						$post_id = $orderDetail->get_id();
						if (!empty($comment)) {
							$data = [
								'comment_post_ID'      => $post_id,
								'comment_content'      => $comment,
								'comment_type'         => 'order_note',
								'comment_parent'       => 0,
								'comment_date'         => null, // получим current_time('mysql')
								'comment_approved'     => 1,
							];

							wp_insert_comment(wp_slash($data));
						}
						if (is_numeric($post_id) && $post_id != false) {
							$fullname = $order->buyer_name;
							$fullname = explode(' ', $fullname);
							update_post_meta($post_id, '_billing_first_name', $fullname[0]);
							update_post_meta($post_id, '_billing_last_name', $fullname[1]);
							update_post_meta($post_id, '_billing_address_1', (empty($order->detail_address) && ($logistics_service_name == "Ozon Логистика" || $logistics_service_name == "доставка силами WB")) ? 'Order ID:' . $order->order_id : $order->detail_address);
							update_post_meta($post_id, '_billing_address_2', '');
							update_post_meta($post_id, '_billing_city', $order->city);
							update_post_meta($post_id, '_billing_state', $order->province);
							update_post_meta($post_id, '_billing_postcode', $order->zip);
							update_post_meta($post_id, '_billing_country', $order->country);
							update_post_meta($post_id, '_billing_email', '');
							update_post_meta($post_id, '_billing_phone', $order->phone_country . '' . $order->mobile_no);

							update_post_meta($post_id, '_shipping_first_name', $fullname[0]);
							update_post_meta($post_id, '_shipping_last_name', $fullname[1]);
							update_post_meta($post_id, '_shipping_address_1', $order->detail_address);
							update_post_meta($post_id, '_shipping_address_2', '');
							update_post_meta($post_id, '_shipping_city', $order->city);
							update_post_meta($post_id, '_shipping_state', $order->province);
							update_post_meta($post_id, '_shipping_postcode', $order->zip);
							update_post_meta($post_id, '_shipping_country', $order->country);
							update_post_meta($post_id, '_billing_email', '');
							update_post_meta($post_id, '_billing_phone', $order->phone_country . '' . $order->mobile_no);

							update_post_meta($post_id, '_order_currency', $order->symbol);
							update_post_meta($post_id, '_billing_address_index', $order->detail_address);
							update_post_meta($post_id, 'ali_order_id', $order->order_id);
							switch ((string)$order->marketplace_id) {
								case "1":
									update_post_meta($post_id, 'is_ali_order', 'yes');
									break;
								case "2":
									update_post_meta($post_id, 'is_ozon_order', 'yes');
									break;
								case "9":
									update_post_meta($post_id, 'is_wb_order', 'yes');
									break;
								default:
									update_post_meta($post_id, 'is_ali_order', 'yes');
									break;
							}

							echo '{"success": true, "comment":"Заказ ' . wp_kses($order->order_id, 'strip') . ' был обновлён в записе ' . wp_kses($post_id, 'strip') . '"}';
							exit();
						}

						echo '{"success": false, "error":"$post_id not correct"}';
						exit();
					} else {
						echo '{"success": false, "error":"offer not exist"}';
						exit();
					}
					exit();
				} else {

					$args = array(
						'body'        => '',
						'timeout'     => '5',
						'redirection' => '5',
						'httpversion' => '1.1',
						'blocking'    => true,
						'headers'     => array(
							'authorization' => 'Token ' . $this->xway_token,
							'content-type' => 'application/json'
						),
						'cookies'     => array(),
					);
					$response = wp_remote_post('https://ali.aliway.ru/api/v1/orders/get/' . $order->order_id . '?key=order_id', $args);
					$order_id = $order->order_id;

					$order = json_decode($response['body']);

					$comment = '';

					switch ($order->order_status) {
						case 'WAIT_SELLER_SEND_GOODS':
						case 'WAIT_SELLER_EXAMINE_MONEY':
						case 'CONFIRMED':
						case 'awaiting_packaging':
							$status = 'wc-processing';
							break;
						case 'EQUIPED':
							$status = 'wc-equiped';
							break;
						case 'SELLER_PART_SEND_GOODS':
							$status = 'wc-part-send';
							break;
						case 'WAIT_BUYER_ACCEPT_GOODS':
						case 'IN_DELIVERY':
						case 'awaiting_deliver':
							$status = 'wc-awaiting';
							break;
						case 'IN_CANCEL':
						case 'REJECTED':
						case 'cancelled':
							$status = 'wc-cancelled';
							$comment = 'Отменён';
							break;
						case 'NOT_EQUIPED':
							$status = 'wc-cancelled';
							$comment = 'Отменён продавцом';
							break;
						case 'FUND_PROCESSING':
						case 'PLACE_ORDER_SUCCESS':
						case 'NEW':
							$status = 'wc-pending';
							break;
						case 'IN_ISSUE':
						case 'IN_FROZEN':
						case 'RISK_CONTROL':
							$status = 'wc-on-hold';
							break;
						case 'DELIVERED':
							$status = 'wc-completed';
							break;
						case 'REFUND':
							$status = 'wc-refunded';
							break;
						case 'FINISH': {
								switch (strtoupper($order->order_end_reason)) {
									case "SELLER_SEND_GOODS_TIMEOUT":
										$status = 'wc-failed';
										$comment = 'Время отправки вышло';
										break;
									case "CANCEL_ORDER_CLOSE_TRADE":
										$status = 'wc-cancelled';
										$comment = 'Заказ отменен клиентом';
										break;
									case "BUYER_ACCEPT_GOODS":
										$status = 'wc-completed';
										$comment = 'Клиент подтвердил получение';
										break;
									case "IN-SALE-RETURN-CLOSE-ORDER":
										$status = 'wc-refunded';
										$comment = 'Полный возврат товара и денег по спору';
										break;
									case "BUYER_CANCEL_NOTPAY_ORDER":
										$status = 'wc-cancelled';
										$comment = 'Покупатель отменил неоплаченный заказ';
										break;
									case "BUYER_PAY_TIMEOUT":
									case "buyer_pay_timeout":
										$status = 'wc-cancelled';
										$comment = 'Покупатель не оплатил заказ вовремя';
										break;
									case "SECURITY_CLOSE":
										$status = 'wc-cancelled';
										$comment = 'Закрыт системой безопасности АлиЭкспресс';
										break;
								}
								break;
							}
					}
					setlocale(LC_ALL, 'ru_RU', 'ru_RU.UTF-8', 'ru', 'russian');

					if ($user_id = email_exists($order->buyer_login_id . '@xway.ru')) {
						$user = $user_id;
					} else {
						$pass = wp_generate_password(6, false);
						$fullname = $order->buyer_name;
						$fullname = explode(' ', $fullname);
						$userdata = array(
							'user_login'    =>   $order->buyer_login_id . '@xway.ru',
							'user_email'    =>   sanitize_email($order->buyer_login_id . '@xway.ru'),
							'user_pass'     =>   $pass,
							'user_url'      =>   "",
							'first_name'    =>   $fullname[0],
							'last_name'     =>   $fullname[1],
							'display_name'	=>	 $order->buyer_signer_fullname,
							'nickname'      =>   $order->buyer_signer_fullname,
							'description'   =>   "",
							'role'			=> 	 "customer",
						);
						$user = wp_insert_user($userdata);
					}

					$new_order = wc_create_order(array(
						'status' => $status,
						'customer_id' => $user
					));

					$escrow_fee_rate = 0;
					$adjust_amount = 0;

					foreach ($order->order_items as $item) {

						$product_id = $this->get_post_by_external_id($item->sku_code); // a product ID or a variation ID
						if ((float)$item->lot_num == null || (float)$item->lot_num == 0) {
							$item->lot_num = 1;
						}
						$new_product_price = (float)$item->product_unit_price_amount / (float)$item->lot_num; // the new product price  <==== <==== <====
						$quantity = $item->product_count; // The line item quantity


						// Get an instance of the WC_Product object

						$product = wc_get_product($product_id);
						// Change the product price
						$product->set_price($new_product_price);
						$new_order->add_product($product, $quantity);


						$shipping = new WC_Order_Item_Shipping();

						$shipping->set_props(array(
							'method_title' => $item->logistics_service_name,
							'method_id'    => '',
							'total'        => wc_format_decimal($item->logistics_amount),
							'taxes'        => array(),
							'order_id'     => $new_order->get_id(),
						));

						$shipping->save();

						$new_order->add_item($shipping);


						if ((float)$item->escrow_fee_rate > $escrow_fee_rate) {
							$escrow_fee_rate = (float)$item->escrow_fee_rate;
						}

						if (count((array)$item->child_order_adjust_amount) > 0) {
							$adjust_amount += (float)$item->child_order_adjust_amount->amount;
						}
					}

					if ($adjust_amount != 0) {
						$item_fee = new WC_Order_Item_Fee();

						$item_fee->set_name(__('Adjustment', 'xway')); // Generic fee name
						$item_fee->set_amount((float)$adjust_amount); // Fee amount
						$item_fee->set_tax_class(''); // default for ''
						$item_fee->set_tax_status('taxable');
						$item_fee->set_total((float)$adjust_amount);
						$item_fee->calculate_taxes((float)$adjust_amount);

						$new_order->add_item($item_fee);
					}

					update_post_meta($new_order->get_id(), 'ali_commission', (float)('-' . $escrow_fee_rate));

					$new_order->calculate_totals();

					$new_order->save();

					if ($status == "wc-completed" || $status == "wc-processing" || $status == "wc-on-hold") {
						wc_reduce_stock_levels($new_order->get_id());
					}
					$post_id = $new_order->get_id();
					if (!empty($comment)) {
						$data = [
							'comment_post_ID'      => $post_id,
							'comment_content'      => $comment,
							'comment_type'         => 'order_note',
							'comment_parent'       => 0,
							'comment_date'         => null,
							'comment_approved'     => 1,
						];

						wp_insert_comment(wp_slash($data));
					}
					if (is_numeric($post_id) && $post_id != false) {
						$fullname = $order->buyer_name;
						$fullname = explode(' ', $fullname);
						update_post_meta($post_id, '_billing_first_name', $fullname[0]);
						update_post_meta($post_id, '_billing_last_name', $fullname[1]);
						update_post_meta($post_id, '_billing_address_1', $order->detail_address);
						update_post_meta($post_id, '_billing_address_2', '');
						update_post_meta($post_id, '_billing_city', $order->city);
						update_post_meta($post_id, '_billing_state', $order->province);
						update_post_meta($post_id, '_billing_postcode', $order->zip);
						update_post_meta($post_id, '_billing_country', $order->country);
						update_post_meta($post_id, '_billing_email', '');
						update_post_meta($post_id, '_billing_phone', $order->phone_country . '' . $order->mobile_no);

						update_post_meta($post_id, '_shipping_first_name', $fullname[0]);
						update_post_meta($post_id, '_shipping_last_name', $fullname[1]);
						update_post_meta($post_id, '_shipping_address_1', $order->detail_address);
						update_post_meta($post_id, '_shipping_address_2', '');
						update_post_meta($post_id, '_shipping_city', $order->city);
						update_post_meta($post_id, '_shipping_state', $order->province);
						update_post_meta($post_id, '_shipping_postcode', $order->zip);
						update_post_meta($post_id, '_shipping_country', $order->country);
						update_post_meta($post_id, '_billing_email', '');
						update_post_meta($post_id, '_billing_phone', $order->phone_country . '' . $order->mobile_no);

						update_post_meta($post_id, '_order_currency', $order->symbol);
						update_post_meta($post_id, '_billing_address_index', $order->detail_address);
						if (is_numeric($order->order_id) && $order->order_id != false && !empty($order->order_id)) {
							update_post_meta($post_id, 'ali_order_id', $order->order_id);
						} else {
							update_post_meta($post_id, 'ali_order_id', $order_id);
							echo '{"success": false, "error":"$order->order_id не корретный, но заказ создан под ID ' . wp_kses($post_id, 'strip') . '"}';
							exit();
						}
						switch ((string)$order->marketplace_id) {
							case "1":
								update_post_meta($post_id, 'is_ali_order', 'yes');
								break;
							case "2":
								update_post_meta($post_id, 'is_ozon_order', 'yes');
								break;
							case "9":
								update_post_meta($post_id, 'is_wb_order', 'yes');
								break;
							default:
								update_post_meta($post_id, 'is_ali_order', 'yes');
								break;
						}
						echo '{"success": true, "comment":"Заказ ' . wp_kses($order->order_id, 'strip') . ' не был найден и создан записью ' . wp_kses($post_id, 'strip') . '"}';
						exit();
					}
					echo '{"success": false, "error":"$post_id not correct"}';
					exit();
				}
				die();
			}
		}

		//Вывести трек номер
		public function ali_track_number($order)
		{
			if (get_post_meta($order->get_id(), 'is_ali_order', true) == "yes") : ?>
				<style>
					.order_track {
						margin-top: 30px;
						float: left;
					}
				</style>
				<script>
					function get_pdf_label(order_id) {
						jQuery.ajax({
							url: ajaxurl,
							type: "POST",
							data: {
								action: "xway_get_pdf_label",
								order_id: order_id,
							},
							success: function(response) {
								console.log(response);
								if (response != "") {
									var url = 'data:application/pdf;base64,' + response;

									var a = document.createElement('A');
									a.href = url;
									a.download = 'order label ' + <?php echo wp_kses($order->get_id(), 'strip') ?>;
									document.body.appendChild(a);
									a.click();
									document.body.removeChild(a);
								} else {
									alert("<?php echo __('Failed to download order label', 'xway-integration') ?>");
								}
							},
						});
						return false;
					}
				</script>
				<h3 class="order_track"><?php echo __('Shipment', 'xway-integration') ?></h3>
				<p class="form-field form-field-wide wc-track">
					<label for="ali-track-number"><?php _e('Track number of the order:', 'xway'); ?></label>
					<input type="text" id="ali-track-number" name="ali_order_track_number" value="<?php echo wp_kses(get_post_meta($order->get_id(), 'ali_order_track_number', true), 'strip') ?>" />
				</p>
				<p class="form-field form-field-wide wc-track">
					<label for="ali-track-url"><?php _e('Link to track the movement of the order:', 'xway'); ?></label>
					<input type="url" id="ali-track-url" name="ali_order_track_url" value="<?php echo wp_kses(get_post_meta($order->get_id(), 'ali_order_track_url', 'strip'), false) ?>" />
				</p>
				<p class="form-field form-field-wide wc-track">
					<label for="ali-send_type"><?php _e('Shipping status:', 'xway'); ?></label>
					<select name="ali_send_type" id="ali-send_type">
						<option disabled value="">Не указано</option>
						<option value="full"><?php _e('Order has been sent in full', 'xway'); ?></option>
						<option value="part"><?php _e('Order sent partially', 'xway'); ?></option>
					</select>
				</p>
				<p class="form-field form-field-wide wc-track">
					<a href="#" onclick="get_pdf_label(<?php echo wp_kses(get_post_meta($order->get_id(), 'ali_order_id', true), 'strip') ?>);" id="get_pdf_label"><?php _e('Download receiving order label', 'xway'); ?></a>
				</p>
			<?php endif;
		}

		//Получение этикетки заказа
		public function xway_get_pdf_label()
		{
			$order_id = sanitize_text_field(wp_unslash($_POST['order_id']));

			$args = array(
				'body'        => '{
					"trade_order_id": "' . $order_id . '"
				}',
				'timeout'     => '5',
				'redirection' => '5',
				'httpversion' => '1.1',
				'blocking'    => true,
				'headers'     => array(
					'authorization' => 'Token ' . $this->xway_token,
					'content-type' => 'application/json'
				),
				'cookies'     => array(),
			);
			$response = wp_remote_post('https://ali.aliway.ru/api/v1/cainiao-short/get-pdf-parcel-label', $args);

			$response = json_decode($response['body'], true);

			if ($response['success'] == true) {
				echo wp_kses($response['files'][0], 'default');
			}
			wp_die();
		}

		//Отправить трек-информацию на AliExpress
		public function send_track($order_id, $track)
		{
			$_SESSION['xway_error'] = array();

			if (empty($track['ali_send_type'])) {
				return false;
			}


			$postfields = array(
				"order_id" => get_post_meta($order_id, 'ali_order_id', true),
				"track_number" => $track['ali_order_track_number'],
				"track_url" => $track['ali_order_track_url'],
			);

			if ($track['ali_send_type'] == "part") {
				$postfields['ali_send_type'] = $track['ali_send_type'];
			}

			$args = array(
				'body'        => json_encode($postfields),
				'timeout'     => '5',
				'redirection' => '5',
				'httpversion' => '1.1',
				'blocking'    => true,
				'headers'     => array(
					'authorization' => 'Token ' . $this->xway_token,
					'content-type' => 'application/json'
				),
				'cookies'     => array(),
			);
			$response = wp_remote_post('https://ali.aliway.ru/api/v1/orders/ship/', $args);

			$log = $response;

			$response = json_decode($response['body']);
			if ($response->success == true) {
				$data = [
					'comment_post_ID'      => $order_id,
					'comment_content'      => 'Трек номер "' . $track['ali_order_track_number'] . '" отправлен (' . ($track['ali_send_type'] != "part" ? __('All goods sent', 'xway') : __('Goods shipped partially', 'xway')) . ')',
					'comment_type'         => 'order_note',
					'comment_parent'       => 0,
					'comment_date'         => null, // получим current_time('mysql')
					'comment_approved'     => 1,
				];

				wp_insert_comment(wp_slash($data));


				$_REQUEST['order_status'] = ($track['ali_send_type'] != "part" ? 'awaiting' : 'part-send');
				$_POST['order_status'] = ($track['ali_send_type'] != "part" ? 'awaiting' : 'part-send');
				$_GET['order_status'] = ($track['ali_send_type'] != "part" ? 'awaiting' : 'part-send');
			} elseif ($response->error != null) {
				array_push($_SESSION['xway_error'], __('Failed to send track number, API server responded with an error', 'xway') . ':' . $response->error);
			}

			return $response;
		}

		//Вычесть комиссию AliExpress из заказа
		public function ali_take_aliexpress_commission($total, $order)
		{
			if (get_post_meta($order->get_id(), 'is_ali_order', true) == "yes") {
				$total = $total + (float)get_post_meta($order->get_id(), 'ali_commission', true);
			}

			return $total;
		}

		//Вывести комиссию AliExpress из заказа
		public function xway_aliexpress_commission($order_id)
		{
			$order = wc_get_order($order_id);
			if (get_post_meta($order_id, 'is_ali_order', true) == "yes") : ?>
				<tr>
					<td class="label"><?php esc_html_e('Commission AliExpress:', 'xway'); ?></td>
					<td width="1%"></td>
					<td class="total">

						<?php echo wc_price(get_post_meta($order_id, 'ali_commission', true), array('currency' => $order->get_currency())); ?>
					</td>
				</tr>
			<?php endif;
		}

		//Получить ID заказа по external_id
		public function get_post_by_external_id($id = '')
		{

			$result = wp_cache_get('ali_post_by_external_id_' . $id);
			if (false === $result) {

				$args = array(
					'meta_key' => 'xway_external_id',
					'meta_value' => $id,
					'post_status' => 'any',
					'post_type' => 'product'
				);
				$query = new WP_Query($args);
				if ($query->posts) {
					foreach ($query->posts as $key => $post_id) {
						wp_cache_set('ali_post_by_external_id_' . $id, $result);

						return $post_id->ID;
					}
				}
			} else {
				return $result;
			}
		}

		//Получить ID заказа по ID из AliExpress
		public function get_order_by_ali_order_id($id = '')
		{

			$result = wp_cache_get('ali_order_by_ali_order_id_' . $id);
			if (false === $result) {

				$args = array(
					'meta_key' => 'ali_order_id',
					'meta_value' => $id,
					'post_status' => 'any',
					'post_type' => 'shop_order'
				);
				$query = new WP_Query($args);
				if ($query->posts) {
					foreach ($query->posts as $key => $post_id) {
						wp_cache_set('ali_order_by_ali_order_id_' . $id, $result);
						return $post_id->ID;
					}
				}
			} else {
				return $result;
			}
			return false;
		}

		//Сохранить ошибки API
		public function xway_error_save()
		{

			if (count($this->errors) > 0) : ?>
				<div class="notice notice-error">
					<?php foreach ($this->errors as $error) { ?>
						<p><?php echo wp_kses($error, 'strip'); ?></p>
					<?php } ?>
				</div>
<?php
				if (isset($_SESSION['xway_error'])) {
					unset($_SESSION['xway_error']);
				}
			endif;
		}


		//Проверка существование заказа с AliExpress
		public function xway_ali_order_exist($id)
		{
			$result = wp_cache_get('ali_order_exist_' . $id);
			if (false === $result) {
				$args = array(
					'meta_key' => 'ali_order_id',
					'post_type' => 'shop_order',
					'post_status' => 'any',
					'meta_value'   => $id,
				);
				$query = new WP_Query($args);

				if ($query->posts) {
					wp_cache_set('ali_order_exist_' . $id, true);
					return true;
				} else {
					wp_cache_set('ali_order_exist_' . $id, false);
					return false;
				}
			} else {
				return $result;
			}
		}

		//Отправить изменение стока на AliExpress
		public function xway_change_stock($order)
		{

			$items = $order->get_items();
			foreach ($items as $item) {
				//$product_name = $item['name'];
				$product_id = $item['product_id'];
				//$product_variation_id = $item['variation_id']; 
				$stock = get_post_meta($product_id, '_stock', true);
				if (get_post_meta($product_id, 'xway_external_id', true) === "") continue;
				$available = true;
				if ($stock < 1) {
					$available = false;
				}
				$xway = new xway();
				$xway->update_stock(array(
					array(
						'external_id' => get_post_meta($product_id, 'xway_external_id', true),
						'stock' => $stock,
						'available' => $available,
					)
				));
			}
		}

		//Обновление цены на AliExpress
		public function update_price($params = array())
		{
			$_SESSION['xway_error'] = array();

			$items = array();
			foreach ($params as $key => $item) {
				$items[$key]['external_id'] = $item['external_id'];
				$items[$key]['price'] = (float)$item['price'];
				$items[$key]['discount_price'] = (float)$item['discount_price'];
				$items[$key]['available'] = boolval($item['available']);
			}
			$items = json_encode($items);

			$args = array(
				'body'		  => json_encode(
					array(
						'shop_id' => $this->xway_shop_id,
						'id_type' => 'external_id',
						'items' => $item,
					)
				),
				'timeout'     => '5',
				'redirection' => '5',
				'httpversion' => '1.1',
				'blocking'    => true,
				'headers'     => array(
					'authorization' => 'Token ' . $this->xway_token,
					'content-type' => 'application/json'
				),
				'cookies'     => array(),
			);
			$response = wp_remote_post('https://ali.aliway.ru/api/v1/offer_price_batch/update/', $args);

			$log = $response;
			$response = json_decode($response['body']);


			if ($response->status != "200") {
				$log = date('Y-m-d H:i:s') . ' ' . $log;
				array_push($_SESSION['xway_error'], 'Сервер API ответил ошибкой с кодом ' . $response->status);
				return false;
			} elseif (count($response->errors->by_item->error_at_save) > 0) {
				$log = date('Y-m-d H:i:s') . ' ' . $log;
				foreach ($response->errors->by_item->error_at_save[0] as $id => $error) {
					$code_message = '';
					foreach (((array)json_decode(str_replace('\'', '"', $error))) as $message) {
						$code_message = $message[0];
					}
					array_push($_SESSION['xway_error'], 'У товара ' . get_the_title($this->get_post_by_external_id($id)) . ' не удалость обновить цену из-за ошибки: ' . $code_message);
				}
				return false;
			} elseif (!empty($response->errors->price_diff)) {
				$log = date('Y-m-d H:i:s') . ' ' . $log;
				array_push($_SESSION['xway_error'], 'У товара не удалость обновить цену из-за ошибки: ' . $response->errors->price_diff);
				return false;
			}
			return true;
		}

		//Обновление стока на AliExpress
		public function update_stock($params = array())
		{

			$items = array();
			foreach ($params as $key => $item) {
				$items[$key]['external_id'] = $item['external_id'];
				$items[$key]['stock'] = (float)$item['stock'];
				$items[$key]['available'] = boolval($item['available']);
			}
			$items = json_encode($items);

			$args = array(
				'body'		  => json_encode(
					array(
						'shop_id' => $this->xway_shop_id,
						'id_type' => 'external_id',
						'items' => $item,
					)
				),
				'timeout'     => '5',
				'redirection' => '5',
				'httpversion' => '1.1',
				'blocking'    => true,
				'headers'     => array(
					'authorization' => 'Token ' . $this->xway_token,
					'content-type' => 'application/json'
				),
				'cookies'     => array(),
			);
			$response = wp_remote_post('https://ali.aliway.ru/api/v1/offer_stock_batch/update/', $args);

			$log = $response;

			$response = json_decode($response['body']);
			if ($response->status != "200" || count($response->errors->by_item->error_at_save) > 0 || (!empty($response->errors->price_diff))) {
				return false;
			} else {
				return true;
			}
			return false;
		}

		public function xway_save_product_postdata($post_id, $post, $update)
		{

			$api_token  = get_option('xway_token');
			$shop_id	= get_option('xway_shop_id');
			$errors 	= array();
			if (!empty($api_token) && !empty($shop_id)) {
				if (get_post_meta($post_id, 'xway_external_id', true) != "") {
					$current_price = get_post_meta($post_id, '_regular_price', true);
					$new_price = sanitize_text_field(wp_unslash($_REQUEST['_regular_price']));
					$current_sale_price = get_post_meta($post_id, '_sale_price', true);
					$new_sale_price = sanitize_text_field(wp_unslash($_REQUEST['_sale_price']));
					$current_stock_status = get_post_meta($post_id, '_stock_status', true);
					$stock_status = sanitize_text_field(wp_unslash($_REQUEST['_stock_status']));
					$current_stock = get_post_meta($post_id, '_stock', true);
					$stock = sanitize_text_field(wp_unslash((float)$_REQUEST['_stock']));
					switch ($stock_status) {
						case "instock":
							$available = true;
							break;
						default:
							$available = false;
							break;
					}
					if ($stock < 1) {
						$available = false;
					}

					if ($current_price != $new_price || $current_sale_price != $new_sale_price || $current_stock_status != $stock_status) {
						$error = $this->update_price(array(
							array(
								'external_id' => get_post_meta($post_id, 'xway_external_id', true),
								'price' => number_format((float)$new_price, 2, ',', ''),
								'discount_price' => number_format((float)$new_sale_price, 2, '.', ''),
								'available' => $available,
							)
						));
					}
					if ($current_stock != $stock) {

						$error = $this->update_stock(array(
							array(
								'external_id' => get_post_meta($post_id, 'xway_external_id', true),
								'stock' => $stock,
								'available' => $available,
							)
						));
					}

					if (!$error) { //отменить сохранение, если есть ошибки
						$_REQUEST = array();
						$_POST = array();
						$_GET = array();
						return;
					}
				}
			}
		}

		public function xway_save_shop_order_postdata($post_id, $post, $update)
		{
			$api_token  = get_option('xway_token');
			$shop_id	= get_option('xway_shop_id');
			if (!empty($api_token) && !empty($shop_id)) {
				if ("yes" == get_post_meta($post_id, 'is_ali_order', true)) {
					$order_id = get_post_meta($post_id, 'ali_order_id', true);
					$order = wc_update_order(array('order_id' => $post_id));
					if (($order->get_status() == "wc-part-send" || $order->get_status() == "wc-awaiting") && isset($_REQUEST['ali_order_track_number']) && get_post_meta($post_id, 'ali_order_track_number', true) != $_REQUEST['ali_order_track_number']) {
						$this->send_track($post_id, array(
							'ali_order_track_number' => sanitize_text_field(wp_unslash($_REQUEST['ali_order_track_number'])),
							'ali_order_track_url' => sanitize_url($_REQUEST['ali_order_track_url']),
							'ali_send_type' => sanitize_text_field(wp_unslash($_REQUEST['ali_send_type'])),
						));
					}



					$order = wc_update_order(array('order_id' => $post_id));

					if ($_POST['post_status'] == "wc-equiped") {
						//
						$items = array();
						$types = array('line_item');
						// defaults to line_item which allows to get products from order
						$weight = 0;
						// iterating through each order item in the order
						foreach ($order->get_items($types) as $item_id => $item) {


							if (!empty($item->get_variation_id())) {
								$WC_Product = new WC_Product_Variation($item->get_variation_id());
							} else {
								$WC_Product = new WC_Product($item->get_product_id());
							}
							$weight += (float)$WC_Product->get_weight('view') * 1000;

							array_push($items, array(
								'quantity' => $item->get_quantity(),
								'sku' => $WC_Product->get_sku()
							));
						}

						$args = array(
							'body'		  => json_encode(
								array(
									'packages' => array(
										'items' => $items,
										'weight' => $weight,
									),
									"trade_order_id" => $order_id
								)
							),
							'timeout'     => '5',
							'redirection' => '5',
							'httpversion' => '1.1',
							'blocking'    => true,
							'headers'     => array(
								'authorization' => 'Token ' . $this->xway_token,
								'content-type' => 'application/json'
							),
							'cookies'     => array(),
						);
						$response = wp_remote_post('https://ali.aliway.ru/api/v1/cainiao-short/create-logistics-order', $args);
						$log = $response;
						$status = json_decode($response['body'], true);


						if ($status['success'] == true) {
							$track = $status['result']['track_numbers'][0];
							$logistics_order_id = $track['logistics_order_id'];
							$track_number = $track['track_number'];
							$lp_number = $track['lp_number'];
							print_r($status);
							if (!empty($track_number)) {
								update_post_meta($post_id, 'ali_order_track_number', $track_number);
								update_post_meta($post_id, 'ali_order_logistics_order_id', $logistics_order_id);
								update_post_meta($post_id, 'ali_order_lp_number', $lp_number);
							}
						}
					}
				}
			}
		}

		public function add_awaiting_shipment_to_order_statuses($order_statuses)
		{

			$new_order_statuses = array();

			// add new order status after processing
			foreach ($order_statuses as $key => $status) {

				$new_order_statuses[$key] = $status;

				if ('wc-processing' === $key) {
					$new_order_statuses['wc-awaiting'] = __('Waiting for delivery confirmation', 'xway');
				}

				if ('wc-processing' === $key) {
					$new_order_statuses['wc-part-send'] = __('Item shipped partially', 'xway');
				}

				if ('wc-processing' === $key) {
					$new_order_statuses['wc-equiped'] = __('Equiped', 'xway');
				}
			}

			return $new_order_statuses;
		}

		// Создание новых статусов для WooCommerce
		public function register_awaiting_shipment_order_status()
		{
			register_post_status('wc-awaiting', array(
				'label'                     => __('Waiting for delivery confirmation', 'xway'),
				'public'                    => true,
				'exclude_from_search'       => false,
				'show_in_admin_all_list'    => true,
				'show_in_admin_status_list' => true,
				'label_count'               => _n_noop(__('Waiting for delivery confirmation (%s)', 'xway'), 'Waiting for delivery confirmation (%s)')
			));

			register_post_status('wc-part-send', array(
				'label'                     => __('Item shipped partially', 'xway'),
				'public'                    => true,
				'exclude_from_search'       => false,
				'show_in_admin_all_list'    => true,
				'show_in_admin_status_list' => true,
				'label_count'               => _n_noop(__('Item shipped partially (%s)', 'xway'), __('Item shipped partially (%s)', 'xway'))
			));

			register_post_status('wc-equiped', array(
				'label'                     => __('Equiped', 'xway'),
				'public'                    => true,
				'exclude_from_search'       => false,
				'show_in_admin_all_list'    => true,
				'show_in_admin_status_list' => true,
				'label_count'               => _n_noop(__('Equiped (%s)', 'xway'), __('Equiped (%s)', 'xway'))
			));
		}

		public function show_only_ali_orders($query)
		{
			if (isset($_GET['ali_orders'])) {
				$meta_query = array(
					array(
						'key' => 'is_ali_order',
						'value' => 'yes',
					),
				);
				$query->set('meta_query', $meta_query);
			} elseif (isset($_GET['ozon_orders'])) {
				$meta_query = array(
					array(
						'key' => 'is_ozon_order',
						'value' => 'yes',
					),
				);
				$query->set('meta_query', $meta_query);
			} elseif (isset($_GET['wb_orders'])) {
				$meta_query = array(
					array(
						'key' => 'is_wb_order',
						'value' => 'yes',
					),
				);
				$query->set('meta_query', $meta_query);
			}
		}

		public function ali_views($views)
		{

			$args = array(
				'post_type'	=> 'shop_order',
				'meta_key' => 'is_ali_order',
				'meta_value' => 'yes',
				'post_status' => 'any'
			);
			$query = new WP_Query($args);
			if ($query->found_posts > 0) {
				$inserted['ali_orders'] = '<a href="edit.php?post_type=shop_order&ali_orders" ' . (isset($_GET['ali_orders']) ? 'class="current" aria-current="page"' : '') . '>' . __('Orders from AliExpress', 'xway') . ' <span class="count">(' . $query->found_posts . ')</span></a>';
			}
			$args = array(
				'post_type'	=> 'shop_order',
				'meta_key' => 'is_ozon_order',
				'meta_value' => 'yes',
				'post_status' => 'any'
			);
			$query = new WP_Query($args);
			if ($query->found_posts > 0) {
				$inserted['ozon_orders'] = '<a href="edit.php?post_type=shop_order&ozon_orders" ' . (isset($_GET['ozon_orders']) ? 'class="current" aria-current="page"' : '') . '>' . __('Orders from Ozon', 'xway') . ' <span class="count">(' . $query->found_posts . ')</span></a>';
			}
			$args = array(
				'post_type'	=> 'shop_order',
				'meta_key' => 'is_wb_order',
				'meta_value' => 'yes',
				'post_status' => 'any'
			);
			$query = new WP_Query($args);
			if ($query->found_posts > 0) {
				$inserted['wb_orders'] = '<a href="edit.php?post_type=shop_order&wb_orders" ' . (isset($_GET['wb_orders']) ? 'class="current" aria-current="page"' : '') . '>' . __('Orders from Wildberries', 'xway') . ' <span class="count">(' . $query->found_posts . ')</span></a>';
			}
			array_splice($views, 1, 0, $inserted); // splice in at position 3
			return $views;
		}
	}

	$xway = new xway();
}
